@@ -23,22 +23,6 @@ listvar<-sp.obj@data
 listnomvar<-names.attr
 
 
-# Is there a Tk window already open ?
-if(interactive())
-{
- if(!exists("GeoXp.open",envir = baseenv())||length(ls(envir=.TkRoot$env, all.names=TRUE))==2)
- {
-  assign("GeoXp.open", TRUE, envir = baseenv())
- }
- else
- {if(get("GeoXp.open",envir= baseenv()))
-   {stop("Warning : a GeoXp function is already open. Please, close Tk window before calling a new GeoXp function to avoid conflict between graphics")}
-  else
-  {assign("GeoXp.open", TRUE, envir = baseenv())}
- }
-}
-
-
 # Code which was necessary in the previous version
 
  if(is.null(carte) & class.obj=="SpatialPolygonsDataFrame") carte<-spdf2list(sp.obj)$poly
@@ -690,9 +674,26 @@ SGfunc<-function()
 # Reprsentation graphique
 ####################################################
 
-graphique.drift()
-carte(long=long, lat=lat, obs=obs,label=label,symbol=pch[3],carte=carte,nocart=nocart,
-cex.lab=cex.lab,method="",axis=axes)
+# Is there a Tk window already open ?
+if(interactive())
+{
+ if(!exists("GeoXp.open",envir = baseenv())||length(ls(envir=.TkRoot$env, all.names=TRUE))==2)
+ {
+  assign("GeoXp.open", TRUE, envir = baseenv())
+  graphique.drift()
+  carte(long=long, lat=lat, obs=obs,label=label,symbol=pch[3],carte=carte,nocart=nocart,
+  cex.lab=cex.lab,method="",axis=axes)
+ }
+ else
+ {if(get("GeoXp.open",envir= baseenv()))
+   {stop("Warning : a GeoXp function is already open. Please, close Tk window before calling a new GeoXp function to avoid conflict between graphics")}
+  else
+  {graphique.drift()
+   carte(long=long, lat=lat, obs=obs,label=label,symbol=pch[3],carte=carte,nocart=nocart,
+   cex.lab=cex.lab,method="",axis=axes)
+   assign("GeoXp.open", TRUE, envir = baseenv())}
+ }
+}
 
 ####################################################
 # cration de la boite de dialogue
